@@ -906,6 +906,17 @@ impl Rearrange {
             name,
         )
     }
+
+    pub fn conform_to_input_shape(&self, coerce: bool) -> Result<Self, CircuitConstructionError> {
+        Ok(Rearrange::try_new(
+            self.node.clone(),
+            self.spec
+                .conform_to_input_shape(&self.node.info().shape.clone(), coerce)
+                .unwrap(),
+            None,
+        )
+        .unwrap())
+    }
 }
 
 /// Index indexes a node dimwise.
