@@ -104,7 +104,8 @@ pub fn compiler_simp_step(
                 f_wrap!(&einsum_of_permute_merge),
                 f_wrap!(&einsum_merge_scalars),
                 l_wrap!(einsum_pull_removable_axes),
-                f_wrap!(&einsum_permute_to_rearrange),
+                f_wrap!(&|x| einsum_permute_to_rearrange(x)
+                    .map(|z| z.conform_to_input_shape(true).unwrap())),
                 l_wrap!(einsum_pull_scatter),
                 f_wrap!(&einsum_push_down_trace),
                 f_wrap!(&einsum_concat_to_add),
@@ -431,9 +432,8 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
 pub fn add_nest_ltr(add: &Add) -> Add {
     let (l, r) = add.nodes.split_at(2.min(add.nodes.len()));
     let base = Add::new(l.to_vec(), None);
-    r.iter().fold(base, |acc, x| {
-        Add::new(vec![acc.rc(), x.clone()], None)
-    })
+    r.iter()
+        .fold(base, |acc, x| Add::new(vec![acc.rc(), x.clone()], None))
 }
 
 #[pyfunction]
