@@ -391,14 +391,13 @@ pub trait CircuitNode: CircuitNodeInit {
         Self: Clone,
     {
         let axes = self.check_canon_axes(axes)?;
-        Ok(Einsum::try_new(
+        Ok(Einsum::new(
             vec![(self.clone().rc(), (0u8..self.info().rank() as u8).collect())],
             (0u8..self.info().rank() as u8)
                 .filter(|i| !axes.contains(&i))
                 .collect(),
             name,
-        )
-        .unwrap())
+        ))
     }
 
     fn mean(&self, axes: &[i64], name: Option<String>) -> Result<Einsum, CircuitConstructionError>
