@@ -235,13 +235,13 @@ pub fn schedule_dag_strategy_ints(
             SchedulingError::CannotFitInMemory {
                 max_memory: mem_limit,
                 memory_chunks: num_mem_chunks,
-                node_memories: costs_sorted.clone(),
                 string: format!(
                     "limit {} num_chunks {} node_chunks {:?}",
                     oom_fmt(mem_limit),
                     num_mem_chunks,
-                    costs_sorted
+                    &costs_sorted
                 ),
+                node_memories: costs_sorted,
             }
         })
 }
