@@ -26,7 +26,7 @@ use uuid::Uuid;
 ///   `.probs_and_group` attribute of `DiscreteVar`, to prevent several uniform
 ///   `DiscreteVar`s with the same number of samples, from being sampled all
 ///   together.
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct AutoTag {
     #[pyo3(get)]
@@ -127,7 +127,7 @@ impl AutoTag {
     }
 }
 
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct DiscreteVar {
     #[pyo3(get)]
@@ -287,7 +287,7 @@ impl DiscreteVar {
     }
 }
 
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct StoredCumulantVar {
     #[pyo3(get)]
