@@ -288,6 +288,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
 
+    m.add_function(wrap_pyfunction!(circuit::deep_rewrite::basic_simp, m)?)?;
     m.add_function(wrap_pyfunction!(circuit::deep_map_preorder_py, m)?)?;
     m.add_function(wrap_pyfunction!(circuit::deep_map_py, m)?)?;
 
@@ -301,6 +302,20 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
     m.add_function(wrap_pyfunction!(circuit::module_nodes::expand_node_py, m)?)?;
     m.add_function(wrap_pyfunction!(rrfs::save_tensor_rrfs, m)?)?;
     m.add_function(wrap_pyfunction!(rrfs::tensor_from_hash, m)?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit::algebraic_rewrite::einsum_permute_to_rearrange,
+        m
+    )?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit::nb_rewrites::add_elim_removable_axes_weak,
+        m
+    )?)?;
+    m.add_function(wrap_pyfunction!(
+        circuit::nb_rewrites::einsum_elim_removable_axes_weak,
+        m
+    )?)?;
+    m.add_function(wrap_pyfunction!(circuit::nb_rewrites::expand_at_axes, m)?)?;
+    m.add_function(wrap_pyfunction!(rrfs::tensor_from_hash, m)?)?;
     m.add_function(wrap_pyfunction!(
         circuit::module_nodes::inline_all_modules,
         m
