@@ -7,7 +7,7 @@ use crate::{
     smallvec::Sv,
     sv,
     tensor_util::{Shape, TensorInvariantError},
-    util::{dict_to_list, filter_out_idx, is_unique, vec_map_insert, EinInt, ALPHABET},
+    util::{dict_to_list, filter_out_idx, is_unique, vec_map_insert, AxisInt, ALPHABET},
 };
 use itertools::{izip, Itertools};
 use once_cell::sync::Lazy;
@@ -94,7 +94,7 @@ pub fn shape_to_op_shape(shape: &Shape) -> OpShape {
 
 // RInnerInts distribution is heavy tailed - almost everything's 1, but there might be up to 10.
 // we use multiple of 8 bc they're u8 and we'd just pad if we used less
-pub type RInnerInts = Sv<[EinInt; 8]>;
+pub type RInnerInts = Sv<[AxisInt; 8]>;
 pub type RInts = Sv<[RInnerInts; 8]>;
 
 #[pyclass]
@@ -218,20 +218,45 @@ impl RearrangeSpec {
                 .unwrap()
         })
     }
-    pub fn ints_in_inp(&self) -> HashSet<EinInt> {
+    pub fn ints_in_inp(&self) -> HashSet<AxisInt> {
         self.ints_in_inp_it().collect()
     }
-    pub fn ints_in_out(&self) -> HashSet<EinInt> {
+    pub fn ints_in_out(&self) -> HashSet<AxisInt> {
         self.ints_in_out_it().collect()
     }
     #[staticmethod]
     pub fn new_permute(permutation: Vec<usize>) -> RearrangeSpec {
-        RearrangeSpec::new(
-            (0..permutation.len()).map(|i| sv![i as u8]).collect(),
+        Self::new(
+            (0..permutation.len() as u8).map(|i| sv![i]).collect(),
             permutation.iter().map(|i| sv![*i as u8]).collect(),
             sv![OpSize::NONE;permutation.len()],
         )
     }
+    #[staticmethod]
+    pub fn new_permute_combine(permute_combine: RInts) -> RearrangeSpec {
+        let count = permute_combine.iter().flatten().count();
+
+        Self::new(
+            (0..count as u8).map(|i| sv![i]).collect(),
+            permute_combine,
+            sv![OpSize::NONE; count],
+        )
+    }
+    #[staticmethod]
+    pub fn combine_axes_at_end(n: u8, axes_to_combine: RInnerInts) -> RearrangeSpec {
+        Self::new_permute_combine(
+            (0..n)
+                .filter_map(|z| {
+                    if axes_to_combine.contains(&z) {
+                        None
+                    } else {
+                        Some(sv![z])
+                    }
+                })
+                .chain(std::iter::once(axes_to_combine.clone()))
+                .collect(),
+        )
+    }
     pub fn is_permute(&self) -> bool {
         // because there are no squeezes, all we need is no splits/joins and equal input and output rank
         self.input_ints.iter().all(|x| x.len() == 1)
@@ -262,7 +287,7 @@ impl RearrangeSpec {
     pub fn is_valid(&self) -> bool {
         // check each int appears once in input and output
         let result = self.ints_in_inp_it().unique().count() == self.ints_in_inp_it().count()
-            && is_unique(&self.ints_in_out_it().collect::<Vec<EinInt>>());
+            && is_unique(&self.ints_in_out_it().collect::<Vec<AxisInt>>());
         // check each input is in output
         let ints_in_out = self.ints_in_out();
         let ints_in_inp = self.ints_in_inp();
@@ -274,7 +299,7 @@ impl RearrangeSpec {
                 < self.int_sizes.len() as i64;
 
         // check each int only in output has size
-        let only_in_output: Vec<EinInt> = ints_in_out.difference(&ints_in_inp).copied().collect();
+        let only_in_output: Vec<AxisInt> = ints_in_out.difference(&ints_in_inp).copied().collect();
         let result = result
             && only_in_output
                 .iter()
@@ -450,7 +475,7 @@ impl RearrangeSpec {
                 .collect(),
         }
     }
-    pub fn ints_needing_sizes(&self) -> HashSet<EinInt> {
+    pub fn ints_needing_sizes(&self) -> HashSet<AxisInt> {
         self.input_ints
             .iter()
             .filter(|x| x.len() > 1)
@@ -592,7 +617,7 @@ impl RearrangeSpec {
         enum SeqPos {
             First,
             Last,
-            Middle(EinInt),
+            Middle(AxisInt),
         }
         use SeqPos::*;
         let mut prev_idx = vec![None; int_sizes.len()];
@@ -624,7 +649,7 @@ impl RearrangeSpec {
 
         // find the first index in a sequence of repeated indices
         // similar to union find - caches results to make future lookups faster
-        fn find(i: EinInt, prev_idx: &mut [Option<SeqPos>]) -> EinInt {
+        fn find(i: AxisInt, prev_idx: &mut [Option<SeqPos>]) -> AxisInt {
             let mut pos = i;
             while let Some(Middle(prev_pos)) = prev_idx[pos as usize] {
                 pos = prev_pos;
@@ -640,7 +665,7 @@ impl RearrangeSpec {
             first_pos
         }
 
-        let mut map: HashMap<EinInt, EinInt> = HashMap::new();
+        let mut map: HashMap<AxisInt, AxisInt> = HashMap::new();
         let mut new_int_sizes: OpShape = sv![];
         let mut already_accounted_shape = vec![false; int_sizes.len()];
 
@@ -754,14 +779,14 @@ impl RearrangeSpec {
 }
 
 impl RearrangeSpec {
-    pub fn ints_in_inp_it(&self) -> impl Iterator<Item = EinInt> + '_ {
+    pub fn ints_in_inp_it(&self) -> impl Iterator<Item = AxisInt> + '_ {
         self.input_ints.iter().flatten().copied()
     }
-    pub fn ints_in_out_it(&self) -> impl Iterator<Item = EinInt> + '_ {
+    pub fn ints_in_out_it(&self) -> impl Iterator<Item = AxisInt> + '_ {
         self.output_ints.iter().flatten().copied()
     }
     /// All integers must occur in the output, so this is valid
-    pub fn all_ints(&self) -> impl Iterator<Item = EinInt> + '_ {
+    pub fn all_ints(&self) -> impl Iterator<Item = AxisInt> + '_ {
         self.ints_in_out_it()
     }
 
@@ -850,7 +875,7 @@ impl RearrangeSpec {
         }
     }
     /// use at your own risk, this doesn't check whether resulting rearrange is valid
-    pub fn filter_all_tuples(&self, tuples_to_skip: &HashSet<Box<[EinInt]>>) -> RearrangeSpec {
+    pub fn filter_all_tuples(&self, tuples_to_skip: &HashSet<Box<[AxisInt]>>) -> RearrangeSpec {
         RearrangeSpec {
             input_ints: self
                 .input_ints
@@ -940,9 +965,9 @@ impl RearrangeSpec {
     pub fn fuse(inner: &Self, outer: &Self) -> Result<Self, RewriteError> {
         let mut inner_sizes: Vec<u64> = inner.get_unwrapped_sizes();
         let mut outer_sizes: Vec<u64> = outer.get_unwrapped_sizes();
-        let mut inner_replaces: HashMap<EinInt, Vec<EinInt>> = HashMap::new();
-        let mut outer_replaces: HashMap<EinInt, Vec<EinInt>> = HashMap::new();
-        let mut in_to_out: HashMap<EinInt, EinInt> = HashMap::new();
+        let mut inner_replaces: HashMap<AxisInt, Vec<AxisInt>> = HashMap::new();
+        let mut outer_replaces: HashMap<AxisInt, Vec<AxisInt>> = HashMap::new();
+        let mut in_to_out: HashMap<AxisInt, AxisInt> = HashMap::new();
 
         // if possible, split up inner.outer_ints and outer.inner_ints until they have the same size for each subdimension (if not possible, raise error)
         for (outer_ints, inner_ints) in izip!(&outer.input_ints, &inner.output_ints) {
@@ -1029,7 +1054,7 @@ impl RearrangeSpec {
         // update inner.inner_ints and outer.inner_ints using the splits we generated above
         fn expand_dims(
             all_dim_ints: &[RInnerInts],
-            replaces: &mut HashMap<EinInt, Vec<EinInt>>,
+            replaces: &mut HashMap<AxisInt, Vec<AxisInt>>,
         ) -> RInts {
             all_dim_ints
                 .iter()
