@@ -1,4 +1,5 @@
 use crate::hashmaps::FxHashMap as HashMap;
+use crate::lru_cache::TensorCacheRrfs;
 use crate::py_types::PY_UTILS;
 use crate::rrfs::{save_tensor_rrfs, tensor_from_hash_prefix};
 use crate::smallvec::Sv;
@@ -605,7 +606,10 @@ impl TensorIndex {
         result.push(']');
         result
     }
-    pub fn from_bijection_string(string: &str) -> Result<Self, CircuitConstructionError> {
+    pub fn from_bijection_string(
+        string: &str,
+        tensor_cache: &mut Option<TensorCacheRrfs>,
+    ) -> Result<Self, CircuitConstructionError> {
         let er = CircuitConstructionError::ParsingFail {
             string: string.to_owned(),
         };
@@ -626,7 +630,10 @@ impl TensorIndex {
                         return Ok(TensorAxisIndex::Single(int));
                     }
                     if let Some(no_t) = x.strip_prefix('t') {
-                        return tensor_from_hash_prefix(no_t)
+                        return tensor_cache
+                            .as_mut()
+                            .map(|tc| tc.get_tensor(no_t.to_owned()))
+                            .unwrap_or_else(|| tensor_from_hash_prefix(no_t))
                             .map(TensorAxisIndex::Tensor)
                             .map_err(|_e| CircuitConstructionError::TensorHashNotFound {
                                 hash: no_t.to_owned(),
